import sys
import numpy as np
import preparenovonix.novonix_variables as nv
from preparenovonix.novonix_io import get_infile
from preparenovonix.novonix_io import icolumn
from preparenovonix.novonix_io import isnovonix
from preparenovonix.novonix_add import create_reduced_protocol
from preparenovonix.novonix_add import novonix_add_loopnr
from preparenovonix.novonix_add import novonix_add_state
from preparenovonix.novonix_clean import cleannovonix


def prepare_novonix(
    file_to_open, addstate=False, lprotocol=False, overwrite=False, verbose=False
):
    """
    Given a Novonix data file, it prepare it to be handled.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    addstate : boolean
        Yes = add a State column

    lprotocol : boolean
        Yes = add to the initial header a 'reduced' protocol with a step per line and add lines with the loop number and protocol lines

    overwrite: boolean
        Yes : overwrite the input file
        No  : a new file will be created, appending '_prep' at the end of the
              original file file

    verbose : boolean
        Yes = print out some informative statements


    Notes
    -----
    This function returns a clean Novonix file with, possibly,
    3 extra columns.

    Examples
    ---------
    >>> import preparenovonix.novonix_prep as prep
    >>> prep.prepare_novonix('example_data/example_data.csv',addstate=True,lprotocol=True,overwrite=False,verbose= False)
    File example_data_prep.csv has been prepared.
    """

    # Get the input file to work on
    infile, fname = get_infile(file_to_open, overwrite=overwrite)

    # Check if the file has the expected structure for a Novonix file
    answer = isnovonix(infile)
    if not answer:
        sys.exit("STOP Input not from Novonix, {}".format(infile))

    # Clean the Novonix file
    cleannovonix(infile)

    if addstate:
        # Check if the file has a State column and if not, create it
        novonix_add_state(infile, verbose=verbose)

    if lprotocol:
        # Check if the file has a Loop number and Protocol line columns
        # and if not, create it
        novonix_add_loopnr(infile, verbose=verbose)

    print("File {} has been prepared.".format(fname))


if __name__ == "__main__":
    if len(sys.argv) > 1:
        prepare_novonix(
            sys.argv[1], addstate=True, lprotocol=True, overwrite=False, verbose=True
        )



import sys
import preparenovonix.novonix_variables as nv
from preparenovonix.novonix_io import replace_file
from preparenovonix.novonix_io import icolumn


summary = "[Summary]"


def count_tests(infile):
    """
    Given a Novonix data file, count the number of tests
    it contains, by looking for the "[Summary]" line
    that starts all Novonix data files.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    Returns
    -------
    ntests : int
       Number of tests found in the file

    Examples
    ---------
    >>> from preparenovonix.novonix_clean import count_tests
    >>> count_tests('example_data/example_data.csv')
    2
    """

    ntests = 0
    with open(infile, "r") as ff:
        for line in ff:
            if line.strip():  # Jump empty lines
                if summary in line:
                    # Count failed tests for each start of file
                    ntests += 1
    return ntests


def header_data_columns(head_line, data_cols, header):
    """
    Given a Novonix data file, compare the columns
    according to the data and the header.
    If there are more data columns than implied in the header,
    dummy colum names are added (dum#).
    If there are less data columns than implied in the header,
    the program stops.

    Parameters
    -----------
    head_line : string
        Header line with column names.

    data_cols : array of floats
        First line with data

    header: array of strings
        Header. If needed, this header will be modified.

    Examples
    ---------
    >>> from preparenovonix.novonix_clean import header_data_columns
    >>> header = ['# Example header']
    >>> header_data_columns("a,b",[1,2,3],header)
    >>> print(header[-1])
    a,b,dum0

    """

    colnames = head_line.split(",")

    # Remove triling blancks and end of lines
    colnames = [x.strip() for x in colnames]

    # Difference between columns in the header and in the data
    diff = len(data_cols) - len(colnames)

    if diff > 0:
        # Add dum headers
        dums = ""
        for idiff in range(diff):
            dums = dums + ",dum" + str(idiff)

            new_head = str(head_line.rstrip()) + dums + " \n"
        header.append(new_head)

    elif diff < 0:
        sys.exit(
            "STOP novonix_clean.header_data_columns \n"
            + "REASON less data columns than header names \n"
        )
    else:
        header.append(head_line)

    return


def capacity_failed_tests(icapacity, ntests, infile):
    """
    Given a Novonix data file, add up the last capacity
    measurement of each failed test in the file.

    Parameters
    -----------
    icapacity: int
        Column position for the capacity

    ntests : int
        Number of tests in the file

    infile : string
        Input file

    Returns
    -------

    last_capacity: float
        Sum of the last capacity measurements for each test 
        before the last one in the file.

    Examples
    ---------
    >>> from preparenovonix.novonix_clean import capacity_failed_tests
    >>> capacity_failed_tests(2,"example_data/example_data.csv")
    0.4956497995
    """

    last_capacity = 0.0
    if ntests > 1:
        itest = 0
        lastline = " "
        with open(infile, "r") as ff:
            for line in ff:
                if line.strip():
                    if summary in line:
                        itest += 1

                        if ntests > 1 and itest > 1:
                            # Add last capacity of each failed test
                            last_capacity = last_capacity + float(
                                lastline.split(",")[icapacity]
                            )
                        if itest == ntests:
                            return last_capacity
                    lastline = line

    return last_capacity


def cleannovonix(infile):
    """
    Given a Novonix file remove blank lines, correct the header,
    remove failed tests if needed.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    Notes
    -----
    This code returns a cleaned Novonix file

    Examples
    ---------
    >>> from preparenovonix.novonix_clean import cleannovonix
    >>> cleannovonix('example_data/example_data.csv')
    """

    # Count the number of tests
    ntests = count_tests(infile)

    # Find the capacity colum
    icapacity = icolumn(infile, nv.col_c)
    # Find the run time column
    iruntime = icolumn(infile, nv.col_t)

    # Deal with the capacity of the failed tests
    last_capacity = capacity_failed_tests(icapacity, ntests, infile)

    # Start reading the last test
    # Remove blank lines if present in the header
    itest = 0
    header = []
    with open(infile, "r") as ff:
        for line in ff:
            if line.strip():
                if summary in line:
                    itest += 1
                    if itest == ntests:
                        header.append(summary + " \n")
                        break

        # Read until the line with [Data]
        for line in ff:
            if line.strip():
                char1 = line.strip()[0]
                if char1 == "[":
                    cleanhead = line.split("]")
                    header.append(cleanhead[0] + "] \n")
                    if cleanhead[0] == "[Data":
                        break
                else:
                    header.append(line)

        # From the data header, read the column names
        for line in ff:
            if line.strip():
                break

        # Check that the number of data columns matches the header
        line_data1 = ff.readline()
        data = line_data1.split(",")
        header_data_columns(line, data, header)

        # Create a temporary file without blanck lines
        # and new header if needed
        tmp_file = "tmp.csv"
        with open(tmp_file, "w") as tf:
            for item in header:
                tf.write(str(item))

        # Append the data jumping any line with time going backwards
        with open(tmp_file, "a") as tf:
            # Write the first data row
            if ntests > 1:
                # Modify the Capacity column in case of failed tests
                new_capacity = float(data[icapacity]) + float(last_capacity)
                data[icapacity] = str(new_capacity)

                new_line = data[0]
                for col in data[1:]:
                    new_line = new_line + "," + col
                tf.write(new_line)
            else:
                tf.write(line_data1)

            # Write the rest of the data
            last_t = -1.0
            for line in ff:
                columns = line.split(",")
                if float(columns[iruntime]) < last_t:
                    continue
                last_t = float(columns[iruntime])

                if ntests > 1:
                    # Modify the Capacity column in case of failed tests
                    new_capacity = float(columns[icapacity]) + float(last_capacity)
                    columns[icapacity] = str(new_capacity)

                    new_line = columns[0]
                    for col in columns[1:]:
                        new_line = new_line + "," + col
                    tf.write(new_line)
                else:
                    tf.write(line)

        # Replace the input file with the new one
        replace_file(tmp_file, infile, newbigger=False)
    return



import sys, os.path
import numpy as np
from shutil import move, copy
import preparenovonix.novonix_variables as nv


def after_file_name(file_to_open):
    """
    Given a file name return as:
    [file_to_open root]_prep.[file-to_open_ending]

    Parameters
    ----------
    file_to_open : string
        Name of the input file.

    Returns
    --------
    after_file : string
        Full path to the (new) file.

    Examples
    ---------
    >>> from preparenovonix.novonix_io import after_file_name
    >>> after_file_name('example_data/example_data.csv')
    """

    # Extract the path and file name
    dirname, fname = os.path.split(os.path.abspath(file_to_open))

    root = fname.split(".")[0]
    ending = fname.split(".")[1]
    fname = root + "_prep." + ending
    after_file = os.path.join(dirname, fname)

    return after_file


def get_infile(file_to_open, overwrite=False):
    """
    Given a file name return it after dealing
    with possible issues with the path  and 
    copy it if the overwrite flag is set to True.

    Parameters
    ----------
    infile_name : string
        Name of the input file.

    overwrite: boolean
        Yes : overwrite the input file
        No  : a new file will be created, appending '_prep' at the end of the
              original file file

    Returns
    --------
    infile : string
        Path to the (new) file.

    fname : string
        File name withouth the path.

    Examples
    ---------
    >>> from preparenovonix.novonix_io import get_infile
    >>> get_infile('example_data/example_data.csv',overwrite=False)
    """

    # Extract the path and file name
    dirname, fname = os.path.split(os.path.abspath(file_to_open))

    if overwrite:
        infile = os.path.join(dirname, fname)
    else:
        infile = after_file_name(file_to_open)
        # If *prep* file already exists, it will be replaced.
        copy(file_to_open, infile)

    return infile, fname


def isnovonix(infile):
    """
    Given a data file, check if it exists and
    if looks like a Novonix data file, allowing for blank lines
    and commas after the commands due to having open the file in Excel

    Parameters
    ----------
    infile : string
        Name of the input Novonix data file

    Returns
    --------
    answer : boolean
        Yes=the file seems to be a Novonix data file

    Examples
    ---------
    >>> from preparenovonix.novonix_io import isnovonix
    >>> isnovonix('example_data/example_data.csv')
    True
    """

    answer = True

    # Test if the file exists
    if not os.path.isfile(infile):
        answer = False
        print(
            "STOP novonix_io.isnovonix \n"
            + "REASON Input file not found: "
            + str(infile)
            + " \n"
        )
        return answer
    else:
        with open(infile, "r") as ff:
            # Read until different header statement
            keyws = ["Summary", "Novonix", "Protocol", "Data"]
            for keyw in keyws:
                for line in ff:
                    if line.strip():
                        char1 = line.strip()[0]
                        if char1 in nv.numberstr:
                            answer = False
                            print(
                                "STOP novonix_io.isnovonix \n"
                                + "REASON Reached the end of the input file \n"
                                + "       "
                                + str(infile)
                                + ", \n"
                                + "       without the "
                                + keyw
                                + " entry."
                            )
                            return answer
                        else:
                            if keyw in line:
                                break

            # Read until the data starts
            for line in ff:
                if line.strip():
                    char1 = line.strip()[0]
                    if char1 in nv.numberstr:
                        break
                    else:
                        last_line = line.strip()

            # From the data header, read the column names
            colnames = last_line.split(",")

            # Remove triling blancks and end of lines
            colnames = [x.strip() for x in colnames]

            # Check the existance of the "Step Number" column
            if nv.col_step not in colnames:
                answer = False
                print(
                    "STOP novonix_io.isnovonix \n"
                    + 'REASON No "Step Number" colum found in input file \n'
                    + "       "
                    + str(infile)
                    + " \n"
                )
                return answer

            # Check the existance of the "Step time" column
            if nv.col_tstep not in colnames:

                answer = False
                print(
                    "STOP novonix_io.isnovonix \n"
                    + 'REASON No "Step Time" colum found in input file \n'
                    + "       "
                    + str(infile)
                    + " \n"
                )
                return answer

    return answer


def icolumn(infile, column_name):
    """
    Given a Novonix data file, find the column position of the
    given column_name

    Parameters
    -----------
    infile : string
        Name of the input Novonix data file

    column_name : string
        Name of the column name to find

    icolumn : integer
        icolumn = -1 : the column has not been found.
        If icolumn > -1: icolumn is the index of the column.

    Returns
    --------
    icol : integer
       Position of the column with name 'column_name',
       in the range form 0 to the number of columns - 1

    Examples
    ---------
    >>> from preparenovonix.novonix_io import icolumn
    >>> icolumn('example_data/example_data.csv','Step Number')
    2
    """

    icol = -1

    # Check if the file has the expected structure for a Novonix data file
    answer = isnovonix(infile)
    if not answer:
        sys.exit("STOP Input not from Novonix, {}".format(infile))

    with open(infile, "r") as ff:
        # Read until the line with [Data]
        for line in ff:
            if "[Data]" in line:
                break

        # From the data header, read the column names
        line = ff.readline()
        colnames = line.split(",")

        # Find the position of the given column name
        ii = 0
        for col in colnames:
            cs = col.strip()
            if column_name.casefold() == cs.casefold():
                icol = ii
                return icol
            ii += 1

    return icol


def read_column(infile, column_name, outtype="float"):
    """
    Given a Novonix data file, read a column as an array of the
    type given in the variable astype.

    Parameters
    -----------
    infile : string
        Name of the input Novonix data file

    column_name : string
        Name of the column to be read

    outtype : string
        Type of data of the column to be read

    Returns
    --------
    column_data : numpy array of the given type
        Column of interest read as a str

    Examples
    ---------
    >>> from preparenovonix.novonix_io import read_column
    >>> col = read_column('example_data/example_data_prep.csv',
    'Step Number',outtype='int')
    >>> print(col[0])
    0
    """

    # Check if the file has the expected structure for a Novonix data file
    answer = isnovonix(infile)
    if not answer:
        sys.exit("STOP Input not from Novonix, {}".format(infile))

    # Find the position of the given column name
    icol = icolumn(infile, column_name)
    if icol < 0:
        sys.exit(
            "STOP novonix_io.readcolumn \n"
            + "REASON "
            + column_name
            + " columnn \n"
            + "      not found in "
            + str(infile)
            + " \n"
        )

    # Initialise empty list
    column_data = []

    with open(infile, "r") as ff:
        # Read until the data starts
        for line in ff:
            if line.strip():
                char1 = line.strip()[0]
                if char1 in nv.numberstr:
                    break

        # Read first value of the column
        val = line.split(",")[icol].rstrip()
        column_data.append(val)

        # Read the rest of the column of interest
        for line in ff:
            val = line.split(",")[icol].rstrip()
            column_data.append(val)

        # Transform the list into a numpy array
        column_data = np.array(column_data)

        column = column_data.astype(getattr(np, outtype))

    return column


def replace_file(newfile, infile, newbigger=False):
    """
    Replace infile by newfile, testing, if adequate,
    if the new file is larger than the older.

    Parameters
    ----------
    newfile : string
        Name of the new file

    infile : string
        Name of the file to be replaced.

    newbigger: boolean
        Yes : test if the newfile is larger than infile.
        No  : simply move the files

    Examples
    ---------
    >>> from preparenovonix.novonix_io import replace_file
    >>> replace_file("example_data/example_data_prep.csv","example_data/example_data.csv")
    """

    if newbigger:
        # Check that the size of the newfile is
        # bigger than the original infile
        size_original = os.stat(infile).st_size
        size_tmp = os.stat(newfile).st_size
        if size_original > size_tmp:
            sys.exit(
                "STOP novonix_io.replace_file \n"
                + "REASON new file is smaller than the original one \n"
                + "       "
                + str(infile)
            )

    # Replace the input file with the new one
    move(newfile, infile)
    return


def get_format(line):
    """
    Given a file line, establish the format

    Parameters
    -----------
    line : string
        Line of the header

    Returns
    --------
    fmt_space : boolean
        True indicates the  main protocol commands 
        have words separated by spaces, which it is
        assumed to be connected with the presence of a 
        semi-colons in the header lines.

    commands : array of string
        Main protocol commands in the adequate format

    Examples
    ---------
    >>> import preparenovonix.novonix_io as prep
    >>> fmt_space, commands = prep.get_format('[0: Open_circuit_storage:]')
    False
    """

    if ":" in line:
        fmt_space = False
        commands = nv.com_prot
    else:
        fmt_space = True
        commands = []
        for com in nv.com_prot:
            newcom = com.replace("_", " ")
            commands.append(newcom)

    return fmt_space, commands


def get_command(line, fmt_space):
    """
    Given a header line, get the possible command

    Parameters
    -----------
    line : string
        Line of the header

    fmt_space : boolean
        Yes = Novonix format with spaces in the commands

    Returns
    --------
    command : string
        Instruction in the header line

    Examples
    ---------
    >>> import preparenovonix.novonix_io as prep
    >>> command = prep.get_command('[Open circuit storage]',fmt_space=True)
    >>> print(command)
    Open circuit storage
    """

    command = " "

    fw = line.strip()
    # Find commands ignoring left spaces
    if fmt_space:
        command = fw[1:-1]
    else:
        if ":" in fw:
            command = fw.split(":")[1].strip()
        else:
            command = fw[1:-1]
    return command


def get_col_names(infile):
    """
    Given a Novonix data file, read the names of the columns.

    Parameters
    -----------
    infile : string
        Name of the input Novonix data file

    Returns
    --------
    col_names : list
        Names of the columns in the file

    Examples
    ---------
    >>> from preparenovonix.novonix_io import get_col_names
    >>> col_names = get_col_names('example_data/example_data_prep.csv')
    >>> print(col_names[2])
    Step Number
    """

    # Check if the file has the expected structure for a Novonix data file
    answer = isnovonix(infile)
    if not answer:
        sys.exit("STOP Input not from Novonix, {}".format(infile))

    # Initialise the list
    col_names = []

    with open(infile, "r") as ff:
        # Read until the line with [Data]
        for line in ff:
            if "[Data]" in line:
                break

        # Read the column names
        col_names1 = ff.readline().split(",")
        for i, coln1 in enumerate(col_names1):
            coln = coln1.replace("\n", "").replace("\r", "").strip()
            col_names.append(coln)

    return col_names


def get_num_measurements(infile):
    """
    Given a prepared Novonix data file, 
    get the number of measurements

    Parameters
    -----------
    infile : string
        Name of the input Novonix data file

    Returns
    --------
    nmeasurements : int
        Number of measurements

    Examples
    ---------
    >>> from preparenovonix.novonix_io import get_num_measurements
    >>> get_num_measurements('example_data/example_data_prep.csv')
    5752
    """

    # Check if the file has the expected structure for a Novonix data file
    answer = isnovonix(infile)
    if not answer:
        sys.exit("STOP Input not from Novonix, {}".format(infile))

    nmeasurements = 0

    with open(infile, "r") as ff:
        # Read until the data starts
        for line in ff:
            if line.strip():
                char1 = line.strip()[0]
                if char1 in nv.numberstr:
                    break

        # Include first measurement
        nmeasurements += 1

        # Count all the measurements
        for line in ff:
            nmeasurements += 1

    return nmeasurements



import sys, os.path
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.gridspec as gridspec
import preparenovonix.novonix_variables as nv
from preparenovonix.novonix_io import read_column
from preparenovonix.novonix_io import after_file_name
from preparenovonix.novonix_io import icolumn


def plot_vct(before_file, first_loop=0, plot_type="pdf", plot_show=False):
    """
    Given two Novonix data files, pre and post-processing
    with preparenovonix, plot toghether their 
    Voltage, Capacity, Step Number and Loop number versus time

    Parameters
    -----------
    before_file : string
        Name of the pre-processed file

    first_loop : int
        Value for the Loop Number to start showing data

    plot_type : string
        'pdf', 'png', 'jpg' for the format of the saved file

    plot_show : boolean
        True to show the plot.

    Notes
    -----
    This code returns a plot.

    Examples
    ---------
    >>> from preparenovonix.compare import plot_vct
    >>> plot_vct('example_data/example_data.csv','example_data/example_data_prep.csv',first_loop=0,plot_type='pdf',plot_show=True)
    """

    after_file = after_file_name(before_file)
    dirname, fname = os.path.split(os.path.abspath(after_file))
    figname = os.path.join(dirname, "compare_vct." + plot_type)

    # Value of the loop to start the plot from
    val = first_loop

    ### Read the voltage, step number and loop number from
    # the processed file

    a_t = read_column(after_file, nv.col_t, outtype="float")
    a_v = read_column(after_file, nv.col_v, outtype="float")
    a_c = read_column(after_file, nv.col_c, outtype="float")
    a_s = read_column(after_file, nv.col_step, outtype="int")
    a_l = read_column(after_file, nv.loop_col, outtype="int")
    a_p = read_column(after_file, nv.line_col, outtype="int")

    # Find the column positions in the file
    icol_t = icolumn(after_file, nv.col_t)
    icol_v = icolumn(after_file, nv.col_v)
    icol_c = icolumn(after_file, nv.col_c)
    icol_step = icolumn(after_file, nv.col_step)

    ### Read the voltage and step number from the original file
    # Since there are failed tests, these need to be jumped
    data = "[Data]"
    ntests = 0  # Count the number of failed tests
    with open(before_file, "r") as ff:
        for line in ff:
            if line.strip():  # Jump empty lines
                if data in line:
                    # Count failed tests for each start of file
                    ntests += 1
                    idata = 0

    vv = []
    ss = []
    tt = []
    cc = []
    with open(before_file, "r") as ff:
        for line in ff:
            if line.strip():  # Jump empty lines
                if data in line:
                    # Count failed tests for each start of file
                    idata += 1
                    if idata == ntests:
                        # Read the header
                        ff.readline()
                        break
        for line in ff:
            tt.append(line.split(",")[icol_t])
            vv.append(line.split(",")[icol_v])
            cc.append(line.split(",")[icol_c])
            ss.append(line.split(",")[icol_step])

    b_t = np.asarray(tt, dtype=np.float64)
    b_v = np.asarray(vv, dtype=np.float64)
    b_c = np.asarray(cc, dtype=np.float64)
    b_s = np.asarray(ss, dtype=int)

    # Plot
    cols = ["navy", "salmon", "cornflowerblue", "darkred"]
    plt.figure(figsize=(8.0, 11.0))
    gs = gridspec.GridSpec(4, 1)
    gs.update(wspace=0.0, hspace=0.0)
    fs = 15

    # Plot only when the loop starts to be biggeer than val
    ind = np.where(a_l > val)
    if np.shape(ind)[1] < 1:
        print("WARNING compare.plot_vct: not enough data to be plotted")
        return

    astart = ind[0][0]
    first_a_t = a_t[astart]
    ind = np.where(b_t >= first_a_t)
    bstart = ind[0][0]

    # Loop number
    axl = plt.subplot(gs[3, :])
    axl.set_xlabel(nv.col_t, fontsize=fs)
    axl.set_ylabel(nv.loop_col, fontsize=fs, color=cols[0])

    axl.plot(a_t[astart::], a_l[astart::], cols[0], linewidth=2.5, label="Loop number")

    axp = axl.twinx()
    axp.set_ylabel("Protocol line", fontsize=fs, color=cols[2])
    axp.plot(
        a_t[astart::], a_p[astart::], cols[2], linewidth=2.5, label="Protocol line"
    )

    # Steps
    axs = plt.subplot(gs[2, :], sharex=axl)
    plt.setp(axs.get_xticklabels(), visible=False)
    axs.set_ylabel(nv.col_step, fontsize=fs)

    axs.plot(a_t[astart::], a_s[astart::], cols[0], linewidth=2.5, label="After")
    axs.plot(b_t[bstart::], b_s[bstart::], cols[1], linestyle="--", label="Before")

    # Voltage and capacity vs. time
    axv = plt.subplot(gs[:-2, :], sharex=axl)
    plt.setp(axv.get_xticklabels(), visible=False)
    axv.set_ylabel(nv.col_v, fontsize=fs)

    axv.plot(
        a_t[astart::], a_v[astart::], cols[0], linewidth=2.5, label="Potential after"
    )
    axv.plot(
        b_t[bstart::], b_v[bstart::], cols[1], linestyle="--", label="Potential before"
    )

    leg = axv.legend(loc=2, fontsize=fs - 2)
    ii = 0
    for text in leg.get_texts():
        text.set_color(cols[ii])
        ii += 1
    leg.draw_frame(False)

    axc = axv.twinx()
    axc.set_ylabel(nv.col_c, fontsize=fs)
    axc.plot(
        a_t[astart::], a_c[astart::], cols[2], linewidth=2.5, label="Capacity after"
    )
    axc.plot(
        b_t[bstart::], b_c[bstart::], cols[3], linestyle="--", label="Capacity before"
    )

    leg = axc.legend(loc=4, fontsize=fs - 2)
    ii = 2
    for text in leg.get_texts():
        text.set_color(cols[ii])
        ii += 1
    leg.draw_frame(False)

    plt.savefig(figname)

    if plot_show:
        plt.show()

    print("Plot: {}".format(figname))



import sys
import numpy as np
import preparenovonix.novonix_variables as nv
from preparenovonix.novonix_io import replace_file
from preparenovonix.novonix_io import icolumn
from preparenovonix.novonix_io import read_column
from preparenovonix.novonix_io import get_command
from preparenovonix.novonix_io import get_format


def column_check(infile, col_name, verbose=False):
    """
    Given a cleaned Novonix data file,
    check if the col_name column exists.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    col_name : string
        Name of the column

    verbose : boolean
        True to print information out.

    Returns
    -------
    column_exists : boolean
        True when the column name has been found in the header

    Examples
    ---------
    >>> import preparenovonix.novonix_variables as nv
    >>> import preparenovonix.novonix_add as prep
    >>> prep.column_check('example_data/example_data_prep.csv',nv.col_step)
    True
    """

    icol = icolumn(infile, col_name)
    if icol > -1:
        if verbose:
            print("The file already has the column {}".format(col_name))
        return True
    else:
        return False


def state_check(state):
    """
    Perform tests on the State array

    Parameters
    -----------
    state : array of integers
        State array

    col_name : string
        Name of the column

    Returns
    -------
    answer : boolean
        True when the tests went fine

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> prep.state_check([0,1,2])
    True
    """

    answer = True
    # Check if the first and last state values have adequate values
    if state[0] != 0 or state[-1] != 2:
        answer = False
        print(
            "WARNING novonix_add.state_check \n"
            + "REASON the State column was not properly populated \n"
        )

    # Check that there are the same numbers of 0s and 2s
    izeros = np.where(np.array(state) == 0)
    itwos = np.where(np.array(state) == 2)
    if np.shape(izeros)[1] != np.shape(itwos)[1]:
        answer = False
        print(
            "WARNING novonix_add.state_check \n"
            + "REASON there is a mismath between State=0 and 2 \n"
        )
    return answer


def novonix_add_state(infile, verbose=False):
    """
    Given a cleaned Novonix data file, it adds a 'State' column,
    which mimimcs Basytec format with:

    0 = Start of a measurement type (charge/discharge, etc)
    1 = Regular data point (measuring, no mode change)
    2 = End of cycle (last point of the measurement)

    This values are determined by the change in the 'Step Number' from Novonix
    and the 'Step time', which goes to 0 with each new 'Step'.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    verbose : boolean
        Yes : print out some informative statements

    Notes
    -----
    This code returns a Novonix file with an extra 'State' column.

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> prep.novonix_add_state('example_data/example_data_prep.csv',verbose=True)
    The file example_data/example_data_prep.csv already has a State column
    """

    # Check if the State column already exists
    col_exists = column_check(infile, nv.state_col, verbose=verbose)
    if col_exists:
        return

    # Find the Step Number column
    icol = icolumn(infile, nv.col_step)
    icolt = icolumn(infile, nv.col_tstep)

    # Read the input file
    header = []
    fw = "fw"
    with open(infile, "r") as ff:
        # Read until the line with [Data]
        for line in ff:
            header.append(line)
            fw = line.rsplit()
            if fw[0] == "[Data]":
                break

        # Read the column names and add the 'State' one
        line = ff.readline()
        new_head = str(line.rstrip()) + ", " + nv.state_col + " \n"
        header.append(new_head)

        # Create a temporary file with the new header
        tmp_file = "tmp.csv"
        ihead = 0
        with open(tmp_file, "w") as tf:
            for item in header:
                tf.write(str(item))
                ihead += 1

        # The State column
        state = []
        data = []
        last_step = -99  # Create a starting last state value
        last_t = 99.0  # Create a starting step time value
        steps = []
        # Read the data adding values to the state
        for il, line in enumerate(ff):
            data.append(line)
            step = line.split(",")[icol]
            steps.append(step)
            stime = float(line.split(",")[icolt])
            if step == last_step and stime > last_t:
                state.append(1)
            else:
                state.append(0)

                # Change the previous value
                if len(state) > 1:
                    if state[-2] == 0:
                        if last_t < nv.eps:
                            # Single measurement
                            state[-2] = -1
                        else:
                            # Jump lines affected by software bug and
                            # 2 single measurements in a row
                            # (this can happen when current overshoots)
                            state[-2] = -99
                            if verbose:
                                # Line count starts with 1, thus (il+1)
                                print(
                                    "WARNING line=",
                                    str(il + ihead),
                                    ", last step time=" + str(last_t),
                                    ": Measurement to be nv.ignored",
                                )
                            if state[-3] == -1:
                                # Avoid 2 single measurements in a row
                                state[-3] = -99
                                if verbose:
                                    print(
                                        "WARNING Measurement to be nv.ignored: line=",
                                        str(il - 1 + ihead),
                                        ", last step time=" + str(last_t),
                                    )

                    elif state[-2] == 1:
                        state[-2] = 2
                    else:
                        sys.exit(
                            "STOP function novonix_add_state \n"
                            + "REASON unexpected state \n"
                            + "      "
                            + str(infile)
                            + " \n"
                        )
            last_step = step
            last_t = stime
        state[-1] = 2

        # Check the new column
        check_pass = state_check(state)
        if not check_pass:
            sys.exit("STOP novonix_add.novonix_add_state \n" + str(infile) + " \n")

        # Write the new data to the temporary file
        with open(tmp_file, "a") as tf:
            ii = 0
            for idata in data:
                if state[ii] > -99:
                    new_line = str(idata.rstrip()) + "," + str(state[ii]) + "\n"
                    tf.write(new_line)
                ii += 1

        # Replace the input file by the tmp_file,
        # which should be bigger.
        replace_file(tmp_file, infile, newbigger=True)

        if verbose:
            print("{} contains now a State column".format(infile))
    return


def select_com_val(index):
    """
    Establish the selection statement for the
    one or two possible command values (com_val#)

    Parameters
    -----------
    index : integer
       Index within the above arrays com_val1, nv.com_val2

    Returns
    --------
    sel : string
       String with the selection using either one or two values

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> prep.select_com_val(2)
    'np.logical_or(step == com_val1[index], step == nv.com_val2[index])'
    """
    if nv.com_val2[index] is None:
        sel = "step == nv.com_val1[index]"
    else:
        sel = "np.logical_or(step == nv.com_val1[index], step == nv.com_val2[index])"
    return sel


def read_reduced_protocol(infile, verbose=False):
    """
    Given a cleaned Novonix data file, read the reduced protocol
    if it exists.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    verbose : boolean
        Yes = print out some informative statements

    Returns
    --------
    protocol : list
        List with the reduced protocol.

    protocol_exists : bool
        False if there is no reduced protocol.

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> protocol, continue_reading = prep.read_reduced_protocol(
    'example_data/example_data_prep.csv',verbose=True)
    >>> print(continue_reading)
    False
    """

    protocol_exists = False

    protocol = [nv.protocol_first]

    fw = " "
    with open(infile, "r") as ff:
        while fw != protocol[0].strip():
            line = ff.readline()
            fw = line.strip()
            if fw == "[Data]":
                return protocol, protocol_exists

        while fw != nv.end_rprotocol.strip():
            line = ff.readline()
            fw = line.strip()
            if fw == "[Data]":
                sys.exit(
                    "STOP novonix_add.read_reduced_protocol \n"
                    + "REASON: line [End reduced protocol] not found \n"
                    + "       "
                    + str(infile)
                    + " \n"
                )
            protocol.append(line)
            protocol_exists = True
        return protocol, protocol_exists


def protocol_check(infile, istate, verbose=False):
    """
    Given a cleaned Novonix data file
    and the expected number of different measurements from the header,
    check if the obtained protocol is reasonable given the data.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    istate : int
        Number of measurements derived from reading the protocol

    verbose : boolean
        True to print information statements

    Returns
    -------
    viable_prot : boolean
        True when the reduced protocol is adequate given the data

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> prep.protocol_check('example_data/example_data_prep.csv',103)
    True
    """

    viable_prot = True

    # Test that the number of protocol lines taking into account repetitions.
    step_number = read_column(infile, nv.col_step, outtype="int")
    state_number = read_column(infile, nv.state_col, outtype="int")

    # Find the number of different steps (CC-CV is considered one)
    ind = np.where(
        (state_number < 1) & (step_number != nv.CCCV_CVc) & (step_number != nv.CCCV_CVd)
    )
    uniq_step = np.shape(ind)[1]
    minus1 = np.shape(
        np.where(
            (state_number == -1)
            & (step_number != nv.CCCV_CVc)
            & (step_number != nv.CCCV_CVd)
        )
    )[1]
    if verbose:
        print(
            "Unique steps = {} (step=-1: {}), Steps from protocol = {}".format(
                uniq_step, minus1, istate
            )
        )

    if istate > uniq_step and verbose:
        print(
            "WARNING function reduced_protocol \n"
            + "REASON Mismatched protocol ("
            + str(istate)
            + ") and steps ("
            + str(uniq_step)
            + "): "
            + str(infile)
            + " \n"
        )

    if istate < uniq_step:
        viable_prot = False
        if verbose:
            print(
                "WARNING function reduced_protocol \n"
                + "REASON Less protocol lines ("
                + str(istate)
                + ") than actual steps ("
                + str(uniq_step)
                + "): \n"
                + str(infile)
                + " \n"
            )

    return viable_prot


def rep_info_not_fmtspace(line, fmt_space):
    """
    Given a line from a file with format: fmt_space=False,
    get the number or repetitions and 
    the number of steps that are being repeated

    Parameters
    -----------
    line : string
        Line with data from a Novonix data file

    fmt_space : boolean
        This function only works if this variable is False

    Returns
    -------
    ncount : int
        Number of repetitions in a loop

    nstep : int
        Number or steps in a loop

    unexpected : boolean
        Flag to state if an unexpected syntax has been encountered

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> ncount, nstep, unexpected = prep.rep_info_not_fmtspace('[5: Repeat: 24 time(s) Node count: 4]',False)
    >>> print(ncount, nstep, unexpected)
    24 4 False
    """

    if fmt_space:
        sys.exit("STOP novonix_add.rep_info_not_fmtspace: wrong format")

    unexpected = False
    linestrip = line.strip()
    if ":" in linestrip:
        fw = linestrip.split(":")
        if len(fw) < 4:
            unexpected = True
        else:
            ncount = int(fw[2].strip().split()[0])
            nstep = int(fw[3][:-1])
    else:
        unexpected = True

    return ncount, nstep, unexpected


def create_end_repeat(nstep, iline, protocol, inrepeat):
    """
    Given a line from a file with format:
    add to the protocol an [# : End repeat nstep steps :] line

    Parameters
    -----------
    nstep : int
        Number of steps in a loop

    iline : int
        Counter for protocol lines

    protocol : array of strings
        Array with the reduced protocol

    inrepeat : boolean
        True if inside a loop

    Returns
    -------
    protocol : array of strings
        Array with the reduced protocol,
        with the added new line

    inrepeat : boolean
        Set to False by this function

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> protocol, inrepeat = prep.create_end_repeat(34,1,['Example'],True)
    >>> print(protocol[-1],inrepeat)
    [0 : End Repeat 34 steps :]
     False
    """

    new_line = "[" + str(iline - 1) + " : End Repeat " + str(nstep) + " steps :"
    protocol.append(new_line + "] \n")

    inrepeat = False

    return protocol, inrepeat


def create_reduced_protocol(infile, verbose=False):
    """
    Given a Novonix data file, get a reduced protocol
    with one command per line.

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    verbose : boolean
        Yes = print out some informative statements

    Returns
    --------
    protocol : list
        List with the reduced protocol

    viable_prot : bool
        False if there was a problem creating the reduced protocol.

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> protocol, viable_prot = prep.create_reduced_protocol('example_data/example_data_prep.csv',verbose=True)
    >>> print(viable_prot)
    True
    >>> print(protocol[0],protocol[-1])
    [Reduced Protocol]
     [End Reduced Protocol]
    """

    # Read the reduced protocol if it already exists
    protocol, protocol_exists = read_reduced_protocol(infile, verbose=verbose)
    if protocol_exists:
        return protocol, protocol_exists

    # Initialize the protocol array
    protocol = [nv.protocol_first]

    # Header line counter
    ih = 0

    # Create the reduced protocol (if it does not already exist)
    with open(infile, "r") as ff:
        # Read until the protocol starts
        for line in ff:
            ih += 1
            if "[Protocol]" in line:
                break

        # Read until the first character is '[' and compare to the commands
        continue_reading = True
        while continue_reading:
            line = ff.readline()
            ih += 1

            if line[0] == "[":
                # Skip the operating limits if present
                if line.split()[0] != "[Protocol" and line.split()[0] != "[End":
                    continue_reading = False

        # Establish which format is this file
        fmt_space, commands = get_format(line)

        # Set counters to 0 for the number of lines, steps,
        # 'State' blocks (0,1,...,1,2), commands and subcommands
        iline = 0
        new_line = " "
        istep = 0
        nstep = 0
        inrepeat = False
        istate = 0
        command = " "
        isub = 0
        doneendrepeat = False

        # Read until the end of the protocol
        while "[End Protocol]" not in line:
            command = get_command(line, fmt_space)

            if command in commands or command.split()[0] == "Repeat":
                iline += 1
                if iline > 1 and not doneendrepeat:
                    # Append the new line
                    protocol.append(new_line + "] \n")
                    new_line = " "

                    # Create an 'End Repeat line if needed
                    if istep == nstep and inrepeat and not doneendrepeat:
                        iline += 1
                        protocol, inrepeat = create_end_repeat(
                            nstep, iline, protocol, inrepeat
                        )
                        new_line = " "
                doneendrepeat = False

                # Get the number of repetitions and repeated steps
                if command.split()[0] == "Repeat":
                    if inrepeat:
                        sys.exit(
                            "STOP novonix_add.create_reduced_protocol \n"
                            + "    at header line: "
                            + str(ih)
                            + " \n"
                            + "REASON code not set to handle nested loops \n"
                            + "       "
                            + str(infile)
                            + " \n"
                        )

                    # Get the number of repetitions and steps within the loop
                    unexpected = False
                    if fmt_space:
                        line = ff.readline()
                        ih += 1
                        fw = line.strip()
                        if fw[1:-1].split()[0] == "Repeat":
                            ncount = int(fw[1:-1].split()[2])
                        else:
                            unexpected = True
                        line = ff.readline()
                        ih += 1
                        fw = line.strip()
                        if fw[1:-1].split()[0] == "Step":
                            nstep = int(fw[1:-1].split()[2])
                        else:
                            unexpected = True
                    else:
                        ncount, nstep, unexpected = rep_info_not_fmtspace(
                            line, fmt_space
                        )

                    if unexpected:
                        sys.exit(
                            "STOP novonix_add.create_reduced_protocol \n"
                            + "REASON unexpected protocol syntax \n"
                            + "       "
                            + str(infile)
                            + " \n"
                        )

                    new_line = (
                        "[" + str(iline) + " : Repeat " + str(ncount) + " times :"
                    )
                    istep = 0
                    inrepeat = True
                else:
                    if inrepeat:
                        istep += 1
                        istate = istate + ncount
                    else:
                        istate += 1

                    new_line = (
                        "[" + str(iline) + " : " + command.replace(" ", "_") + " : "
                    )

            elif inrepeat and (
                command == nv.increment1
                or command == nv.increment1.replace(" ", "_")
                or command == nv.increment2
                or command == nv.increment2.replace(" ", "_")
            ):
                # Substract any Increment step
                nstep = nstep - 1

            else:
                # Append subcommands without brackets,
                # separated by semicoloms
                isub += 1
                subcommand = command

                if subcommand not in nv.ignore:
                    if isub == 1:
                        new_line = new_line + subcommand
                    else:
                        new_line = new_line + ";" + subcommand

                if subcommand.strip().casefold() == nv.endrepeat.casefold():
                    # Append the last line
                    protocol.append(new_line + "] \n")
                    iline += 1

                    # Create an 'End Repeat line for files with fmt_space=True
                    # which have an endrepeat statement
                    protocol, inrepeat = create_end_repeat(
                        nstep, iline + 1, protocol, inrepeat
                    )

                    new_line = " "
                    doneendrepeat = True

            # Continue reading
            line = ff.readline()
            ih += 1

        # Add last line if not done already
        ilast = int(protocol[-1].split()[0].split("[")[1])

        if iline > ilast:
            protocol.append(new_line + "] \n")

            # Create an 'End Repeat line if needed
            if istep == nstep and inrepeat:
                new_line = (
                    "[" + str(iline + 1) + " : End Repeat " + str(nstep) + " steps :"
                )
                protocol.append(new_line + "] \n")
                new_line = " "

        protocol.append(nv.end_rprotocol)

        # Test the obtained protocol
        viable_prot = protocol_check(infile, istate, verbose=verbose)

    return protocol, viable_prot


def novonix_add_loopnr(infile, verbose=False):
    """
    Given a cleaned Novonix data file, it adds a 'Loop number' column,
    with monotonically increasing numbers and
    the protocol line corresponding to a given measurement.

    Measurements that are not being repeated are assinged: Loop number=0.

    This values are determined by the change in the 'State' and
    the protocol description in the header

    Parameters
    -----------
    infile : string
        Name of the input Novonix file

    verbose : boolean
        Yes = print out some informative statements

    Notes
    -----
    This code returns a Novonix file with two extra columns.

    Examples
    ---------
    >>> import preparenovonix.novonix_add as prep
    >>> prep.novonix_add_loopnr('example_data/example_data_prep.csv',verbose=True)
    The file already has the column Loop number
    """

    # Check if the file already has the new Loop column
    col_exists = column_check(infile, nv.loop_col, verbose=verbose)
    if col_exists:
        return

    # Get the reduced protocol as a list
    protocol, viable_prot = create_reduced_protocol(infile, verbose=verbose)

    # Read the Step_Number column
    steps = read_column(infile, nv.col_step, outtype="int")

    # Read the States and the set of measurements for a single state
    states = read_column(infile, nv.state_col, outtype="int")
    izeros, = np.where(np.logical_or(states == 0, states == -1))
    itwos, = np.where(np.logical_or(states == 2, states == -1))

    linenr = np.zeros(shape=len(steps), dtype=int)
    linenr.fill(-999.0)
    loopnr = np.zeros(shape=len(steps), dtype=int)
    loopnr.fill(-999.0)

    if viable_prot:
        loopnr.fill(0)

        # Prepare to loop over the protocol lines
        prot = protocol[1:-1]
        iprot = 0
        inrepeat = False
        com_repeat = []
        ntimes = 0
        itimes = 0
        iloop = 0
        last_step = -1

        # Loop over the measurement sets
        for iz, it in zip(izeros, itwos):
            # Step of the Data Subset
            step = steps[iz]

            # Read protocol line
            continue_reading = True
            while continue_reading:
                command = prot[iprot].split(":")[1].strip()
                if command.split(" ")[0].strip() == "Repeat":
                    inrepeat = True
                    com_repeat = []
                    iloop += 1
                    itimes = 0
                    irstep = 0
                    ntimes = int(command.split(" ")[1].strip())

                    iprot += 1
                    first_rep = iprot + 1
                    firstend = True

                elif command.split(" ")[0].strip() == "End":
                    nrstep = int(command.split(" ")[2].strip())
                    if firstend:
                        irstep = 0
                        itimes += 1
                        iloop += 1
                        firstend = False

                    # Check that the array of repeated steps has the expected lenght
                    if len(com_repeat) != nrstep:
                        sys.exit(
                            "STOP function novonix_add_loopnr \n"
                            + "REASON array of repeated steps has an unexpected length \n"
                            + "       "
                            + str(len(com_repeat))
                            + " != "
                            + str(nrstep)
                            + " \n"
                            + "       "
                            + str(infile)
                            + " \n"
                        )

                    # Repeated commands
                    index = nv.com_prot.index(com_repeat[irstep])
                    sel = select_com_val(index)
                    if sel:
                        if (step == nv.CCCV_CVc or step == nv.CCCV_CVd) and (
                            last_step == nv.CCCV_CCc or last_step == nv.CCCV_CCd
                        ):
                            # 1 protocol line for CC-CV
                            last_step = -1
                            irstep = irstep - 1

                        # Compare the protocol and step values
                        linenr[iz : it + 1] = first_rep + irstep
                        loopnr[iz : it + 1] = iloop
                        continue_reading = False

                    # Adequately deal with the reduced protocol counters
                    if itimes == ntimes - 1 and irstep == nrstep - 1:
                        # Last command of the last repetition: reset
                        iprot += 1
                        inrepeat = False
                        itimes = 0
                        ntimes = 0
                        nrstep = 0
                        first_rep = 0
                    elif itimes < ntimes - 1 and irstep == nrstep - 1:
                        # Last command within a repetition
                        irstep = 0
                        itimes += 1
                        iloop += 1
                    else:
                        irstep += 1

                elif command in nv.com_prot[:-1]:
                    index = nv.com_prot.index(command)
                    sel = select_com_val(index)
                    if sel:
                        if (step == nv.CCCV_CVc or step == nv.CCCV_CVd) and (
                            last_step == nv.CCCV_CCc or last_step == nv.CCCV_CCd
                        ):
                            # 1 protocol line for CC-CV
                            last_step = -1
                            iprot = iprot - 1
                        else:
                            if inrepeat:
                                com_repeat.append(command)

                        linenr[iz : it + 1] = iprot + 1
                        if inrepeat:
                            loopnr[iz : it + 1] = iloop
                        continue_reading = False

                    iprot += 1
                    if iprot == len(prot):
                        continue_reading = False
                else:
                    sys.exit(
                        "STOP function novonix_add_loopnr \n"
                        + "REASON unexpected command in reduced protocol \n"
                        + "       "
                        + str(infile)
                        + " \n"
                    )
            last_step = step

    # Create a temporary file with the new header
    header = []
    fw = "fw"
    tmp_file = "tmp.csv"
    with open(infile, "r") as ff:
        # Read until the line with [End Protocol]
        while fw != "[Data]":
            line = ff.readline()
            header.append(line)
            fw = line.split()[0]

        # Create a temporary file with the header as it was
        with open(tmp_file, "w") as tf:
            for item in header[:-1]:
                tf.write(str(item))

        # Add the reduced protocol
        with open(tmp_file, "a") as tf:
            for item in protocol:
                tf.write(str(item))
            # Add [Data] line
            tf.write(str(header[-1]))

        # Read the column names and add the 'Loop' and 'Line'
        line = ff.readline()
        new_head = str(line.rstrip()) + ", " + nv.line_col + ", " + nv.loop_col + " \n"
        with open(tmp_file, "a") as tf:
            tf.write(str(new_head))

        # Write the data + 2 new columns in the temporary file
        for ii, line in enumerate(ff):
            # Write the new data to the temporary file
            with open(tmp_file, "a") as tf:
                new_line = (
                    line.rstrip() + "," + str(linenr[ii]) + "," + str(loopnr[ii]) + "\n"
                )
                tf.write(new_line)

        # Replace the input file by the tmp_file,
        # which should be bigger.
        replace_file(tmp_file, infile, newbigger=True)

        if verbose:
            print(
                "{} contains now the columns Loop number and Protocol line".format(
                    infile
                )
            )

    return



eps = 1.0e-5

# Novonix columns
col_step = "Step Number"
col_tstep = "Step Time (h)"
col_t = "Run Time (h)"
col_v = "Potential (V)"
col_c = "Capacity (Ah)"

state_col = "State (0=Start 1=Regular 2=End -1=Single Measurement)"
loop_col = "Loop number"
line_col = "Protocol Line (it refers to the reduced protocol)"

# Commands in Protocols
increment1 = "Increment the cycle counter"
increment2 = "Increment cycle counter"
endrepeat = "End repeat"

ignore = [
    "End storage",
    "End charge",
    "End discharge",
    endrepeat,
    increment1,
    increment2,
    "End increment",
    "Trip conditions",
    "End trip conditions",
    "Save conditions",
    "End save conditions",
    "Operating limits",
    "End operating limits",
    "Emergency limits",
    "End emergency limits",
]

com_prot = [
    "Open_circuit_storage",
    "Constant_current_charge",
    "Constant_current_discharge",
    "CC-CV_charge",
    "CC-CV_discharge",
    "Repeat",
]

protocol_first = "[Reduced Protocol] \n"
end_rprotocol = "[End Reduced Protocol] \n"

# Novonix step values
OCV = 0
CCc = 1
CCd = 2  # in some cases this is also set to 1
CCCV_CCc = 7
CCCV_CVc = 8
CCCV_CCd = 9
CCCV_CVd = 10

com_val1 = [OCV, CCc, CCd, CCCV_CCc, CCCV_CCd]
com_val2 = [None, None, CCc, CCCV_CVc, CCCV_CVd]

numberstr = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "-"]



import pathlib
from setuptools import find_packages, setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.rst").read_text()

desc = (
    "Clean and add extra information to data "
    + "produced by the battery cyclers from Novonix."
)

# This call to setup() does all the work
setup(
    name="preparenovonix",
    version="1.0.3",
    packages=find_packages(exclude=("tests",)),
    description=desc,
    long_description=README,
    long_description_content_type="text/x-rst",
    url="https://github.com/BatLabLancaster/preparenovonix",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,
    install_requires=[
        "numpy>=1.15",
        # Note: Matplotlib is loaded for test plot
        "matplotlib>=3.0",
    ],
)
